/*
 * audio_out_raw32.c
 * Copyright (C) 2010 Peter Belkner <pbelkner@snafu.de>
 * Copyright (C) 2000-2002 Michel Lespinasse <walken@zoy.org>
 * Copyright (C) 1999-2000 Aaron Holtzman <aholtzma@ess.engr.uvic.ca>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <inttypes.h>

#include <a52dec/config.h>
#include <a52dec/a52.h>
#include <a52dec/audio_out.h>
#include "audio_out_internal.h"

static int raw32_setup (ao_instance_t * _instance, int sample_rate, int * flags,
                      sample_t * level, sample_t * bias)
{
    raw32_instance_t * instance = (raw32_instance_t *) _instance;

    if ((instance->set_params == 0) && (instance->sample_rate != sample_rate))
        return 1;
    instance->sample_rate = sample_rate;

    *flags = instance->flags;
    *level = 1;
    *bias = 3145728.0;

    return 0;
}

static int raw32_play (ao_instance_t * _instance, int flags, sample_t * _samples)
{
    raw32_instance_t * instance = (raw32_instance_t *) _instance;
	int32_t *int32_samples = instance->int32_samples+instance->offs;
	int offs=instance->offs + 2*256;
    int size = offs * sizeof (int32_t);

	if (size<=instance->max_size) {
#ifdef LIBA52_DOUBLE
      double * samples = _samples;
#else
#error "22"
      double samples[256 * 2];
      int i;

      for (i = 0; i < 256 * 2; i++)
          samples[i] = _samples[i];
#endif

      instance->set_params = 0;
      double2s32_2 (samples, int32_samples);
      s32_LE (int32_samples, 2);
	  instance->offs = offs;
      instance->size = size;

      return 0;
	}
	else
	  return -1;
}

static void raw32_close (ao_instance_t * _instance)
{
}

static ao_instance_t * raw32_open (int flags)
{
    raw32_instance_t * instance;

    instance = malloc (sizeof (raw32_instance_t));
    if (instance == NULL)
        return NULL;

    instance->ao.setup = raw32_setup;
    instance->ao.play = raw32_play;
    instance->ao.close = raw32_close;

    instance->sample_rate = 0;
    instance->set_params = 1;
    instance->flags = flags;
    instance->offs = 0;
    instance->size = 0;
    instance->max_size = 0;
    instance->int32_samples = NULL;

    return (ao_instance_t *) instance;
}

ao_instance_t * ao_raw32_open (void)
{
    return raw32_open (A52_STEREO);
}

ao_instance_t * ao_raw32dolby_open (void)
{
    return raw32_open (A52_DOLBY);
}
