/*
 * ffmpeg_stub.h
 * Copyright (C) 2010 Peter Belkner <pbelkner@snafu.de>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef __FFMPEG_STUB_H__
#define __FFMPEG_STUB_H__

#include <libavformat/avformat.h>
#include <libswscale/swscale.h>

extern int ffmpeg_stub_valid;

AVCodec *get_mp3_decoder();
AVCodec *get_ac3_decoder();
AVCodec *get_flac_decoder();
AVCodec *get_mpc7_decoder();

extern struct ffmpeg_vs {
  typeof(swscale_version) *swscale_version;
  typeof(avformat_version) *avformat_version;
  typeof(avcodec_version) *avcodec_version;
  typeof(avutil_version) *avutil_version;
} ffmpeg_vs;

#ifdef __DYNAMIC__
#define swscale_version (*ffmpeg_vs.swscale_version)
#define avformat_version (*ffmpeg_vs.avformat_version)
#define avcodec_version (*ffmpeg_vs.avcodec_version)
#define avutil_version (*ffmpeg_vs.avutil_version)
#endif

extern struct av_stub {
  // libavutil
  typeof(av_free) *free;
  typeof(av_malloc) *malloc;
  typeof(av_mallocz) *mallocz;
  typeof(av_rescale_q) *rescale_q;

  // libavcodec
  AVCodec *ac3_decoder;
  AVCodec *mp3_decoder;
  AVCodec *flac_decoder;
  AVCodec *mpc7_decoder;
  typeof(av_dup_packet) *dup_packet;
  typeof(av_free_packet) *free_packet;
  typeof(av_picture_copy) *picture_copy;
  typeof(av_get_bits_per_sample_format) *get_bits_per_sample_format;

  // libavformat
  typeof(av_iformat_next) *iformat_next;
  typeof(av_close_input_file) *close_input_file;
  typeof(av_find_stream_info) *find_stream_info;
  typeof(av_metadata_get) *metadata_get;
  typeof(av_open_input_file) *open_input_file;
  typeof(av_read_frame) *read_frame;
  typeof(av_register_all) *register_all;
  typeof(av_seek_frame) *seek_frame;
  typeof(avformat_seek_file) *format_seek_file;
  typeof(dump_format) *dump_format;
  typeof(url_set_interrupt_cb) *url_set_interrupt_cb;
} av;

#ifdef __DYNAMIC__
// libavutil
#define av_free (*av.free)
#define av_malloc (*av.malloc)
#define av_mallocz (*av.mallocz)
#define av_rescale_q (*av.rescale_q)

// libavcodec
//#define ac3_decoder_p (av.ac3_decoder)
//#define mp3_decoder_p (av.mp3_decoder)
//#define flac_decoder_p (av.flac_decoder)
//#define mpc7_decoder_p (av.mpc7_decoder)
#define ac3_decoder (*av.ac3_decoder)
#define mp3_decoder (*av.mp3_decoder)
#define flac_decoder (*av.flac_decoder)
#define mpc7_decoder (*av.mpc7_decoder)
#define av_dup_packet (*av.dup_packet)
#define av_free_packet (*av.free_packet)
#define av_picture_copy (*av.picture_copy)
#define av_get_bits_per_sample_format (*av.get_bits_per_sample_format)

// libavformat
#define av_iformat_next (*av.iformat_next)
#define av_close_input_file (*av.close_input_file)
#define av_find_stream_info (*av.find_stream_info)
#define av_metadata_get (*av.metadata_get)
#define av_open_input_file (*av.open_input_file)
#define av_read_frame (*av.read_frame)
#define av_register_all (*av.register_all)
#define av_seek_frame (*av.seek_frame)
#define avformat_seek_file (*av.format_seek_file)
#define dump_format (*av.dump_format)
#define url_set_interrupt_cb (*av.url_set_interrupt_cb)
#endif

extern struct avcodec_stub {
  // libavcodec
  typeof(avcodec_alloc_frame) *alloc_frame;
  typeof(avcodec_close) *close;
  typeof(avcodec_decode_audio3) *decode_audio3;
  typeof(avcodec_decode_video2) *decode_video2;
  typeof(avcodec_find_decoder) *find_decoder;
  typeof(avcodec_open) *open;
} avcodec;

#ifdef __DYNAMIC__
#define avcodec_alloc_frame (*avcodec.alloc_frame)
#define avcodec_close (*avcodec.close)
#define avcodec_decode_audio3 (*avcodec.decode_audio3)
#define avcodec_decode_video2 (*avcodec.decode_video2)
#define avcodec_find_decoder (*avcodec.find_decoder)
#define avcodec_open (*avcodec.open)
#endif

extern struct avpicture_stub {
  // libavcodec
  typeof(avpicture_alloc) *alloc;
  typeof(avpicture_deinterlace) *deinterlace;
  typeof(avpicture_free) *free;
} avpicture;

#ifdef __DYNAMIC__
#define avpicture_alloc (*avpicture.alloc)
#define avpicture_deinterlace (*avpicture.deinterlace)
#define avpicture_free (*avpicture.free)
#endif

extern struct sws_stub {
  // libswscale
  typeof(sws_freeContext) *freeContext;
  typeof(sws_getContext) *getContext;
  typeof(sws_scale) *scale;
} sws;

#ifdef __DYNAMIC__
#define sws_freeContext (sws.freeContext)
#define sws_getContext (sws.getContext)
#define sws_scale (sws.scale)
#endif

int ffmpeg_stub_load(const char *path, char *p);
int ffmpeg_stub_unload();

#endif // __FFMPEG_STUB_H__
