/*
 * vout.cpp
 * Copyright (C) 2010 Peter Belkner <pbelkner@snafu.de>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include <Winamp/wa_ipc.h>
#include "vout.h"

// http://forums.winamp.com/printthread.php?s=399b97dbab7c564d7ab880ebef98d9cd&threadid=168742
// http://www.comp.leeds.ac.uk/vision/software/libRTImage/doc/libRTImage/html/AVISource_8cpp_8bak-source.html

#define VIDEO_MEDIATYPE VIDEO_MAKETYPE('Y','V','1','2')

vout_t *vout_get(HWND hWindow)
{
  return (vout_t *)SendMessage(hWindow,WM_USER,0,IPC_GET_IVIDEOOUTPUT);
}

void vout_open(vout_t *vo, int width, int height, double aspectratio)
{
  vo->open(width,height,0,aspectratio,VIDEO_MEDIATYPE);
}

void vout_close(vout_t *vo)
{
  vo->close();
}

void vout_draw(vout_t *vo,
    unsigned char* yBaseAddr, long yRowBytes,
    unsigned char* vBaseAddr, long vRowBytes,
    unsigned char* uBaseAddr, long uRowBytes)
{
  YV12_PLANES yv12planes;

  yv12planes.y.baseAddr=yBaseAddr;
  yv12planes.y.rowBytes=yRowBytes;
  yv12planes.v.baseAddr=vBaseAddr;
  yv12planes.v.rowBytes=vRowBytes;
  yv12planes.u.baseAddr=uBaseAddr;
  yv12planes.u.rowBytes=uRowBytes;

  vo->draw(&yv12planes);
}

void vout_set_caption(vout_t *vo, char *buf)
{
  vo->extended(VIDUSER_SET_INFOSTRING,(int)buf, 0);
}

HWND vout_get_hwnd(vout_t *vo)
{
  return (HWND)vo->extended(VIDUSER_GET_VIDEOHWND, 0, 0);
}
